import { createYSEnv, global } from "./evaluate.js";
import { concat } from "./psmap.js";
import { load, moduleEval } from "./load.js";
import { map } from "./data.js";
import { run as $run } from "./deps.js";
export function createPlatformScript(globals) {
    let env = lazy(() => {
        let ext = globals ? globals(platformscript) : map({});
        return createYSEnv(concat(global, ext));
    });
    function run(block) {
        return $run(() => block(env()));
    }
    let platformscript = {
        run,
        call(fn, arg, options) {
            return run((env) => env.call(fn, arg, options));
        },
        eval(value, bindings) {
            return run((env) => env.eval(value, bindings));
        },
        moduleEval(value, url) {
            return run((env) => moduleEval({ source: value, location: url, env }));
        },
        load(location, base) {
            return run((env) => load({ location, base, env }));
        },
    };
    return platformscript;
}
function lazy(create) {
    let thunk = () => {
        let value = create();
        thunk = () => value;
        return value;
    };
    return () => thunk();
}
//# sourceMappingURL=platformscript.js.map