export function createPSMap() {
    return {
        type: "map",
        value: new Map(),
    };
}
export function concat(parent, child) {
    return {
        type: "map",
        value: new Map([...parent.value.entries(), ...child.value.entries()]),
    };
}
export function* map(fntor, fn) {
    let entries = [];
    for (let [k, v] of fntor.value.entries()) {
        entries.push([k, yield* fn(v)]);
    }
    return { type: "map", value: new Map(entries) };
}
export function lookup(key, map) {
    if (map.type !== "map") {
        return { type: "nothing", value: void 0 };
    }
    for (let entry of map.value.entries()) {
        let [k, value] = entry;
        if (k.value === key) {
            return { type: "just", value };
        }
    }
    return { type: "nothing", value: void 0 };
}
export function lookup$(key, map) {
    if (map.type !== "map") {
        throw new Error(`cannot lookup a value from non-map type '${map.type}'`);
    }
    let maybe = lookup(key, map);
    if (maybe.type === "nothing") {
        throw new Error(`expected map to contain key '${key}', but it did not`);
    }
    return maybe.value;
}
export function exclude(keys, map) {
    return {
        type: "map",
        value: new Map({
            *[Symbol.iterator]() {
                for (let [key, value] of map.value.entries()) {
                    if (!keys.includes(key.value.toString())) {
                        yield [key, value];
                    }
                }
            },
        }),
    };
}
//# sourceMappingURL=psmap.js.map