import { yaml } from "./deps.js";
import * as data from "./data.js";
export function read(source) {
    let result = yaml.parseDocument(source, { keepSourceTokens: true });
    let [error] = result.errors;
    if (error) {
        throw error;
    }
    return readNode(result.contents);
}
function readNode(node) {
    if (yaml.isScalar(node)) {
        switch (typeof node.value) {
            case "number":
                return literal(node, {
                    type: "number",
                    value: node.value,
                });
            case "boolean":
                return literal(node, {
                    type: "boolean",
                    value: node.value,
                });
            case "string":
                return literal(node, {
                    type: "string",
                    value: node.value,
                });
            default:
                throw new Error(`unknown YAML node type '${typeof node.value}'`);
        }
    }
    else if (yaml.isMap(node)) {
        return literal(node, {
            type: "map",
            value: new Map(node.items.map((item) => {
                //TODO: revisit what is a map key
                let key = readNode(item.key);
                let value = readNode(item.value);
                return [key, value];
            })),
        });
    }
    else if (yaml.isSeq(node)) {
        return literal(node, {
            type: "list",
            value: node.items.map((item) => readNode(item)),
        });
    }
    else if (!node) {
        return data.string("");
    }
    else {
        throw new Error(`unrecognized YAML node: '${node.toString()}`);
    }
}
export function isLiteral(value) {
    return !!value && yaml.isNode(value.node);
}
export function derive(parent, child) {
    if (isLiteral(parent)) {
        literal(parent.node, child);
    }
    return child;
}
function literal(node, value) {
    return Object.defineProperty(value, "node", {
        enumerable: false,
        value: node,
    });
}
//# sourceMappingURL=read.js.map