"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.moduleEval = exports.load = void 0;
const deps_js_1 = require("./deps.js");
const psmap_js_1 = require("./psmap.js");
const evaluate_js_1 = require("./evaluate.js");
const recognize_js_1 = require("./recognize.js");
const data = __importStar(require("./data.js"));
function* load(options) {
    let { location, base, env } = options;
    let url = typeof location === "string" ? new URL(location, base) : location;
    let content = yield* read(url);
    let source = (0, evaluate_js_1.parse)(content);
    return yield* moduleEval({ source, location: url, env });
}
exports.load = load;
function* moduleEval(options) {
    let { location, source, env = (0, evaluate_js_1.createYSEnv)() } = options;
    let url = typeof location === "string" ? new URL(location) : location;
    let mod = {
        url,
        source,
        value: source,
        imports: [],
    };
    if (source.type !== "map") {
        return mod;
    }
    let scope = data.map({});
    let imports = (0, psmap_js_1.lookup)("$import", source);
    let rest = (0, psmap_js_1.exclude)(["$import"], source);
    if (imports.type === "just") {
        if (imports.value.type !== "map") {
            throw new Error(`imports must be specified as a mapping of names: URL but was ${imports.value.type}`);
        }
        for (let [names, loc] of imports.value.value.entries()) {
            if (names.type !== "string") {
                throw new Error(`imported symbols should be a string, but was ${names.type}`);
            }
            if (loc.type !== "string") {
                throw new Error(`import location should be a url string, but was ${loc.type}`);
            }
            let bindings = matchBindings(names.value);
            let dep = yield* load({
                location: loc.value,
                base: url.toString(),
                env,
            });
            mod.imports.push({
                module: dep,
                bindings,
            });
            for (let binding of bindings) {
                let name = binding.alias ?? binding.name;
                let value;
                if (binding.all) {
                    value = dep.value;
                }
                else if (dep.value.type !== "map") {
                    throw new Error(`tried to import a name from ${dep.url}, but it is not a 'map'. It is a ${dep.value.type}`);
                }
                else {
                    let result = (0, psmap_js_1.lookup)(binding.name, dep.value);
                    if (result.type === "nothing") {
                        throw new Error(`module ${dep.url} does not have a member named '${binding.name}'`);
                    }
                    else {
                        value = result.value;
                    }
                }
                scope.value.set(data.string(name), value);
            }
        }
    }
    mod.value = data.map({});
    let expanded = (0, recognize_js_1.recognize)(rest);
    if (expanded.type !== "map") {
        mod.value = yield* env.eval(expanded, scope);
    }
    else {
        for (let [key, value] of expanded.value.entries()) {
            let evaluated = yield* env.eval(value, scope);
            scope.value.set(key, evaluated);
            mod.value.value.set(key, yield* env.eval(value, scope));
        }
    }
    return mod;
}
exports.moduleEval = moduleEval;
function* read(url) {
    if (url.protocol === "file:") {
        return yield* (0, deps_js_1.expect)(Deno.readTextFile(url));
    }
    else if (url.protocol.startsWith("http")) {
        let signal = yield* (0, deps_js_1.useAbortSignal)();
        let response = yield* (0, deps_js_1.expect)(fetch(url, { signal }));
        if (!response.ok) {
            throw new Error(`${url}: ${response.status} ${response.statusText}`);
        }
        return yield* (0, deps_js_1.expect)(response.text());
    }
    else {
        throw new Error(`cannot load module from ${url}: unsupported protocol '${url.protocol}'`);
    }
}
function matchBindings(spec) {
    return spec.trim().split(/\s*,\s*/).map((spec) => {
        let splatMatch = spec.match(/(.+)<<$/);
        if (splatMatch) {
            return {
                name: splatMatch[1],
                all: true,
            };
        }
        else {
            return {
                name: spec,
                all: false,
            };
        }
    });
}
//# sourceMappingURL=load.js.map