"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.derive = exports.isLiteral = exports.read = void 0;
const deps_js_1 = require("./deps.js");
const data = __importStar(require("./data.js"));
function read(source) {
    let result = deps_js_1.yaml.parseDocument(source, { keepSourceTokens: true });
    let [error] = result.errors;
    if (error) {
        throw error;
    }
    return readNode(result.contents);
}
exports.read = read;
function readNode(node) {
    if (deps_js_1.yaml.isScalar(node)) {
        switch (typeof node.value) {
            case "number":
                return literal(node, {
                    type: "number",
                    value: node.value,
                });
            case "boolean":
                return literal(node, {
                    type: "boolean",
                    value: node.value,
                });
            case "string":
                return literal(node, {
                    type: "string",
                    value: node.value,
                });
            default:
                throw new Error(`unknown YAML node type '${typeof node.value}'`);
        }
    }
    else if (deps_js_1.yaml.isMap(node)) {
        return literal(node, {
            type: "map",
            value: new Map(node.items.map((item) => {
                //TODO: revisit what is a map key
                let key = readNode(item.key);
                let value = readNode(item.value);
                return [key, value];
            })),
        });
    }
    else if (deps_js_1.yaml.isSeq(node)) {
        return literal(node, {
            type: "list",
            value: node.items.map((item) => readNode(item)),
        });
    }
    else if (!node) {
        return data.string("");
    }
    else {
        throw new Error(`unrecognized YAML node: '${node.toString()}`);
    }
}
function isLiteral(value) {
    return !!value && deps_js_1.yaml.isNode(value.node);
}
exports.isLiteral = isLiteral;
function derive(parent, child) {
    if (isLiteral(parent)) {
        literal(parent.node, child);
    }
    return child;
}
exports.derive = derive;
function literal(node, value) {
    return Object.defineProperty(value, "node", {
        enumerable: false,
        value: node,
    });
}
//# sourceMappingURL=read.js.map