"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.recognize = void 0;
const read_js_1 = require("./read.js");
const data = __importStar(require("./data.js"));
function recognize(value) {
    if (value.type === "string") {
        let expressions = matchTemplate(value.value);
        if (expressions.length > 0) {
            return {
                type: "template",
                value,
                expressions,
            };
        }
        else if (value.quote) {
            return value;
        }
        else {
            let match = matchReference(value.value);
            if (match) {
                return {
                    type: "ref",
                    value,
                    key: match.key,
                    path: match.path,
                };
            }
            else {
                return value;
            }
        }
    }
    else if (value.type === "map") {
        let entries = [...value.value.entries()];
        let [first] = entries;
        if (!first) {
            return value;
        }
        else {
            let [firstKey, firstValue] = first;
            let fnmatch = String(firstKey.value).match(/^\(\s*(.*)\)\s*=>$/);
            if (fnmatch) {
                let param = fnmatch[1];
                let body = recognize(firstValue);
                return {
                    type: "fn",
                    param: { name: param },
                    value: {
                        type: "platformscript",
                        head: firstKey,
                        body,
                    },
                };
            }
            else {
                let $firstKey = recognize(firstKey);
                if ($firstKey.type === "ref") {
                    let [, ...rest] = entries;
                    return {
                        type: "fncall",
                        value: $firstKey,
                        arg: firstValue,
                        rest: {
                            type: "map",
                            value: new Map(rest),
                        },
                        source: value,
                    };
                }
                return (0, read_js_1.derive)(value, {
                    type: "map",
                    value: new Map(entries.map(([k, v]) => {
                        let value = recognize(v);
                        //match method syntax
                        let mmatch = String(k.value).match(/^\s*(.+)\((.*)\)\s*$/);
                        if (mmatch) {
                            let [, key, param] = mmatch;
                            return [data.string(key), {
                                    type: "fn",
                                    param: { name: param },
                                    value: {
                                        type: "platformscript",
                                        head: k,
                                        body: value,
                                    },
                                }];
                        }
                        else {
                            return [k, value];
                        }
                    })),
                });
            }
        }
    }
    else {
        return value;
    }
}
exports.recognize = recognize;
function matchTemplate(value) {
    let valueIdx = 0;
    let exprIdx = 1;
    let regex = /%\(([\s\S]+?)\)/gmd;
    let i = value.matchAll(regex);
    let expressions = [];
    for (let next = i.next(); !next.done; next = i.next()) {
        let match = next.value;
        let expression = recognize((0, read_js_1.read)(match[exprIdx]));
        expressions.push({
            expression,
            //@ts-expect-error RegExpMatchArray#indices not yet in the default TS lib
            range: match.indices[valueIdx],
        });
    }
    return expressions;
}
function matchReference(value) {
    let pathIdx = 1;
    let match = value.match(/^\$(\S+)$/);
    if (match) {
        let [key, ...path] = match[pathIdx].split(".");
        return {
            key,
            path,
        };
    }
}
//# sourceMappingURL=recognize.js.map